require("dotenv").config();
const path = require("path");

const express = require("express");
const mongoose = require("mongoose");
const morgan = require("morgan");
const bodyParser = require("body-parser");
const passport = require("passport");

const app = express();
const db = process.env.MONGODB_URI;

if (process.env.NODE_ENV !== "test")
  mongoose
    .connect(db, {
      useNewUrlParser: true,
      useUnifiedTopology: true,
      useCreateIndex: true,
    })
    .then(() => console.log("Connected to MongoDB successfully!"))
    .catch(err => console.log(err));

// Middleware
app.use(morgan("dev"));
app.use(bodyParser.urlencoded({ extended: true }));
app.use(bodyParser.json());
app.use(passport.initialize());
require("./config/passport")(passport);
app.use(express.static(path.join(__dirname, "client", "build")));

// Routers
const userRouter = require("./routes/user.router");
app.use("/api/users", userRouter);
const diaryRouter = require("./routes/diary.router");
app.use("/api/diaries", diaryRouter);
const diaryEntryRouter = require("./routes/diaryEntry.router");
app.use("/api/entries", diaryEntryRouter);
const flashcardRouter = require("./routes/flashcard.router");
app.use("/api/flashcards", flashcardRouter);
const seedRouter = require("./seeds/index.seeds");
app.use("/api/seed", seedRouter);

// Boilerplate code from https://www.npmjs.com/package/express-joi-validation#error-handling
app.use((err, req, res, next) => {
  if (err && err.error && err.error.isJoi) {
    console.error(err);
    // we had a joi error, let's return a custom 422 json response
    res.status(422).send({
      type: err.type, // will be "query" here, but could be "headers", "body", or "params"
      message: err.error.toString(),
    });
  } else {
    // pass on to another error handler
    next(err);
  }
});

// Error handling; see: https://medium.com/quick-code/using-mocha-for-test-driven-development-in-building-api-with-node-express-c9331cd6171f
app.use((err, req, res, next) => {
  res.locals.message = err.message;
  res.status(err.status || 500);
  console.error(err);
  res.send(err);
});

app.get("*", (req, res) => {
  res.sendFile(path.resolve(__dirname, "./client/build", "index.html"));
});

module.exports = app;
