import React, { useState } from "react";
import { connect } from "react-redux";
import { createNewEntry } from "../../actions/diary_entry_actions";
import ReactMde, { getDefaultToolbarCommands } from "react-mde";
import ReactMarkdown from "react-markdown";
import "react-mde/lib/styles/css/react-mde.css";
import "react-mde/lib/styles/css/react-mde-editor.css";
import "react-mde/lib/styles/css/react-mde-toolbar.css";
import "react-mde/lib/styles/css/react-mde-suggestions.css";
import moment from "moment";
import { sampleTextContent } from "../util/sample_entry_text";
import { fetchAllDueFlashcards } from "../../actions/flashcard_actions";

export function NewEntryForm({
  submitNewEntry,
  diaryId,
  history,
  fetchFlashcardCount,
}) {
  const [content, setContent] = useState("Enter text here!");
  const [date, setDate] = useState(moment().format("YYYY-MM-DD"));
  const [title, setTitle] = useState("");
  const [location, setLocation] = useState("");
  const [mood, setMood] = useState("");
  const [selectedTab, setSelectedTab] = React.useState("write");
  const [sampleText, setSampleText] = useState(false);
  const [storedContent, setStoredContent] = useState("");
  const [storedTitle, setStoredTitle] = useState("");
  const [makeCards, setMakeCards] = useState(true);
  const [detailsOpen, setDetailsOpen] = useState(false);
  const moods = [
    "😄 Happy",
    "😟 Sad",
    "😪 Tired",
    "😡 Angry",
    "🥰 Romantic",
    "🤩 Inspired",
    "😕 Confused",
  ];
  console.log(`getDefault`, getDefaultToolbarCommands());

  const toggleSampleText = e => {
    const text = sampleTextContent;
    if (sampleText) {
      setContent(storedContent);
      setTitle(storedTitle);
      setSampleText(false);
    } else {
      setStoredContent(content);
      setStoredTitle(title);
      setContent(text);
      setTitle("Sample Entry Title");
      setSampleText(true);
    }
  };

  const handleSubmit = e => {
    e.preventDefault();
    const entryData = {
      date,
      content,
    };
    Object.assign(
      entryData,
      title && { title },
      location && { location },
      mood && { mood },
      makeCards && { makeCards }
    );

    submitNewEntry(diaryId, entryData);
    setTimeout(fetchFlashcardCount, 5000);
    history.push(`/diaries/${diaryId}`);
  };

  return (
    <form onSubmit={handleSubmit} className="diary-entry-form">
      <label htmlFor="newEntryTitle">Entry title</label>
      <input
        type="text"
        value={title}
        onChange={e => setTitle(e.target.value)}
        placeholder="Optional"
        id="newEntryTitle"
      />
      <button onClick={toggleSampleText} className="big-button" type="button">
        {sampleText
          ? "Click to restore your own text"
          : "Click here to enter sample text"}
      </button>
      <label htmlFor="newEntryContent">
        Entry content (click "Preview" to see formatted text)
      </label>
      <details
        className="markdown-hints"
        onToggle={() => setDetailsOpen(!detailsOpen)}
      >
        <summary>
          {detailsOpen
            ? "Close this help panel"
            : "Click here to learn how to format your diary entry (with Markdown):"}
        </summary>
        <div className="entry-preview">
          <p>
            <strong>Emphasis</strong>
          </p>
          <p>
            Surround words with an *asterisk* to <em>italicise</em> words.
          </p>
          <p>
            Or two **asterisks** to make them <strong>bold</strong>.
          </p>
          <hr />
          <p>
            <strong>Headings</strong>
          </p>
          <p># Start a line with a '#' to:</p>
          <h1>Make a Heading Like This</h1>
          <p>## Or start a line with two '##' to:</p>
          <h2>Make a sub-heading like this</h2>
          <hr />
          <p>
            <strong>Lists</strong>
          </p>
          <p>* Making a list is easy</p>
          <p>* Just start each line with an asterisk</p>
          <p>- Or a dash / hyphen</p>
          <ul>
            <li>To make a list</li>
            <li>Just like this one</li>
          </ul>
          <hr />
          <p>
            Don't forget to click{" "}
            <button type="button" className="hint-preview-btn" disabled>
              Preview
            </button>{" "}
            to see what your entry looks like so far!
          </p>
        </div>
      </details>
      <ReactMde
        value={content}
        onChange={setContent}
        selectedTab={selectedTab}
        id="newEntryContent"
        onTabChange={setSelectedTab}
        generateMarkdownPreview={markdown =>
          Promise.resolve(
            <ReactMarkdown className="entry-preview" children={markdown} />
          )
        }
        toolbarCommands={[["bold", "italic"], ["unordered-list"]]}
      />
      <label htmlFor="newEntryId">What date did this memory occur?</label>
      <input
        type="date"
        value={date}
        onChange={e => setDate(e.target.value)}
        placeholder="Date"
        id="newEntryId"
        required
      />
      <label htmlFor="newEntryLocation">Where did this memory occur?</label>
      <input
        type="text"
        value={location}
        onChange={e => setLocation(e.target.value)}
        id="newEntryLocation"
        placeholder="Optional"
      />
      <label htmlFor="newEntryMood">How did you feel during this memory?</label>
      <select
        id="newEntryMood"
        onChange={e => setMood(e.target.value)}
        value={mood}
        placeholder="Optional"
      >
        <option value="">None selected</option>
        {moods.map((moodEntry, index) => {
          const moodText = moodEntry.split()[1];
          return (
            <option key={`${moodText}-${index}`} value={moodText}>
              {moodEntry}
            </option>
          );
        })}
      </select>
      <div className="inline-checkbox">
        <label htmlFor="newEntryMakeCards">
          Make flashcards with this entry?
        </label>
        <input
          type="checkbox"
          id="newEntryMakeCards"
          checked={makeCards}
          onChange={e => setMakeCards(!makeCards)}
        />
      </div>
      <button className="big-button" onClick={handleSubmit} type="submit">
        Add New Entry
      </button>
    </form>
  );
}

const mapStateToProps = (state, { match }) => ({
  diaryId: match.params.diaryId,
});

const mapDispatchToProps = dispatch => ({
  submitNewEntry: (diaryId, entryData) =>
    dispatch(createNewEntry(diaryId, entryData)),
  fetchFlashcardCount: () => dispatch(fetchAllDueFlashcards(true)),
});

export default connect(mapStateToProps, mapDispatchToProps)(NewEntryForm);
