const mongoose = require("mongoose");
const DiaryEntry = require("../models/DiaryEntry.model");
const Flashcard = require("../models/Flashcard.model");
const User = require("../models/User.model");
const {
  runCardmakerService,
  runCardsaverService,
} = require("../services/index.services");

module.exports = {
  getUserEntries: async (req, res, next) => {
    try {
      const entries = await DiaryEntry.find({ userId: req.user._id });
      return res.send(entries);
    } catch (error) {
      next(error);
    }
  },
  getDiaryEntries: async (req, res, next) => {
    try {
      const entries = await DiaryEntry.find({ diaryId: req.params.id });
      return res.send(entries);
    } catch (error) {
      next(error);
    }
  },
  getEntry: async (req, res, next) => {
    try {
      const entry = await DiaryEntry.findById(req.params.id);
      if (!entry) return res.status(404).send("Diary entry not found");
      if (entry.userId.toString() !== req.user._id.toString()) {
        return res
          .status(401)
          .send("You are not authorized to view this diary entry");
      } else {
        return res.send(entry);
      }
    } catch (error) {
      next(error);
    }
  },
  createEntry: async (req, res, next) => {
    try {
      const newEntry = new DiaryEntry({
        ...req.body,
        userId: req.user._id,
        diaryId: req.params.id,
      });
      const savedEntry = await newEntry.save();

      // If user has opted to create flashcards, run cardmaker service and save resultant cards
      if (req.body.makeCards) {
        // Send sanitised user data along to cardmaker
        const user = await User.findById(req.user._id);
        const sanitisedUser = Object.assign({}, user._doc);
        delete sanitisedUser.username;
        delete sanitisedUser.firstName;
        delete sanitisedUser.lastName;
        delete sanitisedUser.email;
        delete sanitisedUser.password;
        console.log(`sanitisedUser`, sanitisedUser);
        // Async code runs while sending saved diary entry to user
        runCardmakerService({
          user: sanitisedUser,
          entries: [savedEntry],
        }).then(response => {
          Flashcard.create(response.cardData)
            .then(result =>
              console.log(
                `${result.length} new flashcards saved at ${new Date()}!`
              )
            )
            .catch(err => console.error(err));
        });
      }

      return res.send(savedEntry);
    } catch (error) {
      next(error);
    }
  },
  updateEntry: async (req, res, next) => {
    try {
      const entry = await DiaryEntry.findById(req.params.id);
      if (!entry) return res.status(404).send("Diary entry not found");
      if (entry.userId.toString() !== req.user._id.toString())
        return res.status(401).send("You are not authorized to do this");

      const updatedEntry = await DiaryEntry.findOneAndReplace(
        { _id: entry._id },
        Object.assign(
          {},
          entry.toObject(),
          { ...req.body },
          { updatedAt: Date.now() }
        ),
        { new: true }
      );

      return res.send(updatedEntry);
    } catch (error) {
      next(error);
    }
  },
  deleteEntry: async (req, res, next) => {
    try {
      const entry = await DiaryEntry.findById(req.params.id);
      if (!entry) return res.status(404).send("Diary entry not found");
      if (entry.userId.toString() !== req.user._id.toString())
        return res.status(401).send("You are not authorized to do this");
      await Flashcard.deleteMany({ diaryEntryId: entry.id });
      const deletedEntry = await DiaryEntry.findByIdAndDelete(entry._id);
      return res.send(deletedEntry);
    } catch (error) {
      next(error);
    }
  },
};
