const mongoose = require("mongoose");
const User = require("../models/User.model");
const bcryptjs = require("bcryptjs");
const jsonwebtoken = require("jsonwebtoken");

async function makeToken(user) {
  const payload = {
    id: user.id,
    username: user.username,
    firstName: user.firstName
  };

  return jsonwebtoken.sign(payload, process.env.SECRET, {
    expiresIn: 21600,
  });
}
module.exports.getAllUsers = async (req, res, next) => {
  try {
    const users = await User.find();
    return res.send(users);
  } catch (error) {
    next(error);
  }
};

module.exports.getUserById = async (req, res, next) => {
  try {
    const { id } = req.params;
    if (!mongoose.Types.ObjectId.isValid(id)) {
      return res.status(400).send("Invalid object ID");
    } else {
      const user = await User.findById(id);
      if (user) {
        return res.send(user);
      } else {
        return res.status(404).send("User not found");
      }
    }
  } catch (error) {
    next(error);
  }
};

module.exports.getCurrentlyLoggedInUser = async (req, res, next) => {
  try {
    return res.send(req.user);
  } catch (error) {
    next(error);
  }
};

module.exports.postUser = async (req, res, next) => {
  try {
    // Check for duplicate email or username
    const existingEmailUser = await User.findOne({ email: req.body.email });
    if (existingEmailUser)
      return res
        .status(409)
        .send("A user already exists with this email address");
    const existingUsernameUser = await User.findOne({
      username: req.body.username,
    });
    if (existingUsernameUser)
      return res.status(409).send("A user already exists with this username");

    const newUser = new User({ ...req.body });

    // Create password hash
    const hashSalt = await bcryptjs.genSalt(10);
    const passwordHash = await bcryptjs.hash(newUser.password, hashSalt);
    newUser.password = passwordHash;

    // Save new user
    const savedUser = await newUser.save();

    // Sign in user after registration
    res.send({
      success: true,
      token: "Bearer " + (await makeToken(savedUser)),
    });
  } catch (error) {
    next(error);
  }
};

module.exports.loginUser = async (req, res, next) => {
  try {
    const { email, password } = req.body;

    const foundUser = await User.findOne({ email: email });
    if (!foundUser) return res.status(404).send("User not found");

    const isMatch = await bcryptjs.compare(password, foundUser.password);
    if (!isMatch) return res.status(401).send("Email or password is incorrect");

    res.send({
      success: true,
      token: "Bearer " + (await makeToken(foundUser)),
    });
  } catch (error) {
    next(error);
  }
};

module.exports.updateUserInfo = async (req, res, next) => {
  try {
    const user = await User.findById(req.params.id);

    if (req.user._id != user._id.toString())
      return res.status(401).send("You are not authorized to do this");

    const updatedUser = await User.findOneAndReplace(
      { _id: user._id },
      Object.assign(
        {},
        user.toObject(),
        { ...req.body },
        { updatedAt: Date.now() }
      ),
      { new: true }
    );
    
    return res.send(updatedUser);
  } catch (error) {
    console.log(error);
    next(error);
  }
};
