const mongoose = require("mongoose");
const moment = require("moment");
const Flashcard = require("../models/Flashcard.model");
const { workerData, parentPort } = require("worker_threads");
const { diaryEntries, user } = workerData;
const MostCommonWordsSet = require("./helpers/commonwords.serviceshelper");

const makeSectionsObject = content => {
  const sections = {};
  const regex = /(\n\n)|(\s)/;
  const contentWords = content.split(regex).filter(x => x && x !== " ");
  let section = [];
  let sectionCount = 0;
  for (const word of contentWords) {
    if (word == `\n\n`) {
      section.push(word);
      if (section.length > 1)
        sections[sectionCount++] = {
          type: section[0].startsWith("#") ? "heading" : "paragraph",
          sectionContent: section,
        };
      section = [];
    } else {
      section.push(word);
    }
  }
  return detectLists(sections);
};

const detectLists = sections => {
  const sectionsArr = Object.entries(sections);
  let heading;
  let list = [];
  let index;

  for (let i = 0; i < sectionsArr.length; i++) {
    const section = sectionsArr[i];
    if (section[1].sectionContent[0] == "*") {
      if (
        i > 0 &&
        sectionsArr[i - 1] &&
        sectionsArr[i - 1][1].sectionContent[0].startsWith("#")
      ) {
        heading = sectionsArr[i - 1];
      } else {
        index = section[0];
      }
      list.push(section[1].sectionContent);
      sectionsArr[i] = null;
    } else if (list.length > 0) {
      if (heading) {
        sectionsArr[parseInt(heading[0])] = [
          heading[0],
          {
            type: "list",
            heading: heading[1].sectionContent,
            sectionContent: list,
          },
        ];
      } else {
        sectionsArr[parseInt(index)] = [
          index,
          {
            type: "list",
            heading: null,
            sectionContent: list,
          },
        ];
      }
      heading = null;
      list = [];
      index = null;
    }
  }

  if (list.length > 0) {
    if (heading) {
      sectionsArr[parseInt(heading[0])] = [
        heading[0],
        {
          type: "list",
          heading: heading[1].sectionContent,
          sectionContent: list,
        },
      ];
    } else {
      sectionsArr[parseInt(index)] = [
        index,
        {
          type: "list",
          heading: null,
          sectionContent: list,
        },
      ];
    }
  }

  return Object.fromEntries(sectionsArr.filter(x => x !== null));
};

const parsedDiaryEntries = diaryEntries.map(entry => {
  return Object.assign({}, entry, {
    content: makeSectionsObject(entry.content),
  });
});

const isEndOfSentence = word => {
  return (
    word.endsWith(".") ||
    word.endsWith(".*") ||
    word.endsWith(".**") ||
    word.endsWith("!") ||
    word.endsWith("!*") ||
    word.endsWith("!**") ||
    word.endsWith("?") ||
    word.endsWith("?*") ||
    word.endsWith("?**")
  );
};

const strippedWord = word => {
  return word.match(/\b[\w']+\b/g)[0];
};

const makeSentences = content => {
  const sentences = [];
  let sentence = "";
  for (const word of content) {
    if (isEndOfSentence(word)) {
      sentence += (sentence.length > 0 ? " " : "") + word;
      if (sentence.length > 10) sentences.push(sentence);
      sentence = "";
    } else {
      sentence += (sentence.length > 0 ? " " : "") + word;
    }
  }
  if (sentence.length > 10) sentences.push(sentence);
  return sentences;
};

const makeClozeOfLength = length => {
  let cloze = "";
  for (let i = 0; i < length; i++) {
    cloze += "_";
  }
  return cloze;
};

const makeClozeSentences = sentence => {
  const splitSentence = sentence.split(" ");
  const clozeSentences = [];
  const emphasisedWords = splitSentence
    .filter(
      word =>
        word.length > 2 &&
        ((word.startsWith("*") && word.endsWith("*")) ||
          (word.startsWith("**") && word.endsWith("**")))
    )
    .filter((word, index, self) => self.indexOf(word) === index);
  if (emphasisedWords.length > 0) {
    for (const empWord of emphasisedWords) {
      const wordIndex = splitSentence.indexOf(empWord);
      const wordLength = empWord.replace("*", "").length;
      const splitCloze = [
        ...splitSentence.slice(0, wordIndex),
        makeClozeOfLength(wordLength),
        ...splitSentence.slice(wordIndex + 1),
      ];
      clozeSentences.push(splitCloze.join(" "));
    }
  } else {
    const filteredSentence = splitSentence.filter(
      (word, index, self) => self.indexOf(word) === index
    );
    for (const word of filteredSentence) {
      if (
        !word.startsWith("#") &&
        word.length > 3 &&
        (!MostCommonWordsSet.has(strippedWord(word).toLowerCase()) ||
          strippedWord(word)[0].toLowerCase() !== strippedWord(word)[0])
      ) {
        const wordIndex = splitSentence.indexOf(word);
        const wordLength = word.length;
        const splitCloze = [
          ...splitSentence.slice(0, wordIndex),
          makeClozeOfLength(wordLength),
          ...splitSentence.slice(wordIndex + 1),
        ];
        clozeSentences.push(splitCloze.join(" "));
      }
    }
  }

  return clozeSentences;
};

const makeParagraphCards = (paragraphSection, headingSection = null) => {
  const paragraphCards = [];
  const sentences = makeSentences(paragraphSection.sectionContent);
  for (let i = 0; i < sentences.length; i++) {
    const sentence = sentences[i];
    const contextUp = [...sentences.slice(0, i)].reverse();
    const contextDown = sentences.slice(i + 1);

    const clozeSentences = makeClozeSentences(sentence);
    for (const clozeSentence of clozeSentences) {
      const cardData = {
        type: "fill_the_blanks",
        frontSide: clozeSentence,
        backSide: sentence,
        contextUp,
        contextDown,
      };
      if (headingSection)
        cardData["heading"] = headingSection.sectionContent.join(" ");
      paragraphCards.push(cardData);
    }
  }

  return paragraphCards;
};

const makeListCards = listSection => {
  const listCards = [];
  const sentences = listSection.sectionContent.map(content =>
    content.join(" ")
  );
  for (let i = 0; i < sentences.length; i++) {
    const sentence = sentences[i];
    const contextUp = [...sentences.slice(0, i)].reverse();
    const contextDown = sentences.slice(i + 1);

    const clozeSentences = makeClozeSentences(sentence);
    for (const clozeSentence of clozeSentences) {
      const cardData = {
        type: "fill_the_blanks",
        frontSide: clozeSentence,
        backSide: sentence,
        contextUp,
        contextDown,
      };
      if (listSection.heading)
        cardData["heading"] = listSection.heading.join(" ");
      listCards.push(cardData);
    }
  }

  return listCards;
};

const makeEntryCards = entry => {
  const { diaryId, userId, _id, date } = entry;
  const flashcardData = [];
  const contentArr = Object.values(entry.content);
  let skipNext = false;
  for (let i = 0; i < contentArr.length; i++) {
    if (skipNext) {
      skipNext = false;
      continue;
    }
    const section = contentArr[i];
    if (section.type == "heading") {
      if (contentArr.length > i + 1 && contentArr[i + 1].type == "paragraph") {
        flashcardData.push(...makeParagraphCards(contentArr[i + 1], section));
        skipNext = true;
      } else {
        flashcardData.push(...makeParagraphCards(section));
      }
    } else if (section.type == "list") {
      flashcardData.push(...makeListCards(section));
    } else if (section.type == "paragraph") {
      flashcardData.push(...makeParagraphCards(section));
    }
  }

  return flashcardData.map(card => {
    return Object.assign({}, card, {
      diaryId,
      userId,
      diaryEntryId: _id,
      entryDate: date,
    });
  });
};

const makeLocationCard = entry => {
  const dateOptions = {
    weekday: "long",
    year: "numeric",
    month: "long",
    day: "numeric",
  };
  const dateStr = new Date(entry.date).toLocaleString("en-GB", dateOptions);
  return {
    type: "location",
    frontSide: "Where were you on **" + dateStr + "?** 🗺 🏖 ⛪️ 🌃",
    backSide: entry.location,
    diaryId: entry.diaryId,
    userId: entry.userId,
    diaryEntryId: entry._id,
    entryDate: entry.date,
  };
};

const makeMoodCard = entry => {
  const dateOptions = {
    weekday: "long",
    year: "numeric",
    month: "long",
    day: "numeric",
  };
  const dateStr = new Date(entry.date).toLocaleString("en-GB", dateOptions);
  return {
    type: "mood",
    frontSide: "What was your mood like on **" + dateStr + "**? 😁 😭 😜 🤬",
    backSide: entry.mood,
    diaryId: entry.diaryId,
    userId: entry.userId,
    diaryEntryId: entry._id,
    entryDate: entry.date,
  };
};

const makeDateCards = entry => {
  const dateCards = [];
  for (const section of Object.values(entry.content)) {
    let frontSide;
    if (section.type == "paragraph") {
      frontSide = section.sectionContent.join(" ");
    } else if (section.type == "list") {
      frontSide = section.sectionContent.map(sent => sent.join(" ")).join(" ");
    } else {
      continue;
    }
    const dateOptions = {
      weekday: "long",
      year: "numeric",
      month: "long",
      day: "numeric",
    };
    const dateStr = new Date(entry.date).toLocaleString("en-GB", dateOptions);
    dateCards.push({
      type: "date",
      heading: "When did you make this memory? 🕰 🗓 ⏳",
      frontSide: frontSide,
      backSide: dateStr,
      diaryId: entry.diaryId,
      userId: entry.userId,
      diaryEntryId: entry._id,
      entryDate: entry.date,
    });
  }

  return dateCards;
};

const makeAgeCards = entry => {
  const ageCards = [];
  const userBirthday = moment(user.birthday);
  const entryDate = moment(entry.date);
  const age = entryDate.diff(userBirthday, "years");

  for (const section of Object.values(entry.content)) {
    let frontSide;
    if (section.type == "paragraph") {
      frontSide = section.sectionContent.join(" ");
    } else if (section.type == "list") {
      frontSide = section.sectionContent.map(sent => sent.join(" ")).join(" ");
    } else {
      continue;
    }
    ageCards.push({
      type: "date",
      heading: "How old were you when you made this memory? 👶 🧒 🧑 🧓",
      frontSide: frontSide,
      backSide: `${age} years old`,
      diaryId: entry.diaryId,
      userId: entry.userId,
      diaryEntryId: entry._id,
      entryDate: entry.date,
    });
  }

  return ageCards;
};

const newFlashcardData = [];

for (const entry of parsedDiaryEntries) {
  newFlashcardData.push(...makeEntryCards(entry));
  newFlashcardData.push(...makeDateCards(entry));
  if (entry.location) {
    newFlashcardData.push(makeLocationCard(entry));
  }
  if (entry.mood) {
    newFlashcardData.push(makeMoodCard(entry));
  }
  if (user.birthday) {
    newFlashcardData.push(...makeAgeCards(entry));
  }
}

parentPort.postMessage({ cardData: newFlashcardData });
