const User = require("../../models/User.model");
const Diary = require("../../models/Diary.model");
const DiaryEntry = require("../../models/DiaryEntry.model");
const Flashcard = require("../../models/Flashcard.model");
const { testUser } = require("../datasets/user.testdata");
const { validDiaries } = require("../datasets/diary.testdata");
const { validDiaryEntries } = require("../datasets/diaryentry.testdata");

const flashcardData = {
  type: "fill_the_blanks",
  frontSide: "Et quod est in quaerat amet minus [________] et natus.",
  backSide: "Et quod est in quaerat amet minus **nesciunt** et natus.",
  contextUp: [
    "Nostrum dolor *nihil* inventore aut quas *ipsam* sed.",
    "Ab impedit eos est amet quia.",
    "## Exercitationem ratione tenetur voluptas ut.\n",
  ],
  contextDown: [
    "Hic saepe quo temporibus eos.",
    "Tenetur quis eum et cum dolorum natus corporis placeat aut.",
    "Eius quo harum tenetur minima quis nulla eos.\n",
    "Veniam iusto illo et aperiam **culpa.**",
  ],
};

describe("Flashcard model", () => {
  beforeEach(async function () {
    const user = await new User(testUser).save();
    this.currentTest.user = user;
    const diaryData = validDiaries[0];
    const diary = await new Diary({
      ...diaryData,
      userId: this.currentTest.user._id,
    }).save();
    this.currentTest.diary = diary;
    const entryData = validDiaryEntries[0];
    const entry = await new DiaryEntry({
      ...entryData,
      userId: user._id,
      diaryId: diary._id,
    }).save();
    this.currentTest.entry = entry;
  });

  it("should create new flashcard with valid data", async function () {
    const flashcard = new Flashcard({
      ...flashcardData,
      userId: this.test.user._id,
      diaryId: this.test.diary._id,
      diaryEntryId: this.test.entry._id,
      entryDate: this.test.entry.date
    });

    const savedFlashcard = await flashcard.save();
    expect(savedFlashcard).to.equal(flashcard);
    expect(savedFlashcard).to.have.property("_id");
  });

  it("should fail to create a new flashcard with no diaryId", async function () {
    const flashcard = new Flashcard({
      ...flashcardData,
      userId: this.test.user._id,
      diaryEntryId: this.test.entry._id,
      entryDate: this.test.entry.date
    });

    try {
      await flashcard.save();
      throw new Error("Should not have saved this record!");
    } catch (err) {
      expect(err.message).to.include("validation failed");
    }
  });

  it("should fail to create a new flashcard with no userId", async function () {
    const flashcard = new Flashcard({
      ...flashcardData,
      diaryId: this.test.diary._id,
      diaryEntryId: this.test.entry._id,
      entryDate: this.test.entry.date
    });

    try {
      await flashcard.save();
      throw new Error("Should not have saved this record!");
    } catch (err) {
      expect(err.message).to.include("validation failed");
    }
  });

  it("should fail to create a new flashcard with no diaryEntryId", async function () {
    const flashcard = new Flashcard({
      ...flashcardData,
      userId: this.test.user._id,
      diaryId: this.test.diary._id,
      entryDate: this.test.entry.date
    });

    try {
      await flashcard.save();
      throw new Error("Should not have saved this record!");
    } catch (err) {
      expect(err.message).to.include("validation failed");
    }
  });

  it("should fail to create a new flashcard with no entryDate", async function () {
    const flashcard = new Flashcard({
      ...flashcardData,
      userId: this.test.user._id,
      diaryId: this.test.diary._id,
      diaryEntryId: this.test.entry._id,
    });

    try {
      await flashcard.save();
      throw new Error("Should not have saved this record!");
    } catch (err) {
      expect(err.message).to.include("validation failed");
    }
  });
});
