const Diary = require("../../models/Diary.model");
const User = require("../../models/User.model");
const request = require("supertest");
const app = require("../../app");
const { testUser, validUsers } = require("../datasets/user.testdata");
const {
  registerUserAndGetToken,
} = require("../helpers/user_and_session.testhelper");
const TestDiaries = require("../datasets/diary.testdata");
const { expect } = require("chai");

describe("api/diaries", () => {
  describe("GET /", () => {
    beforeEach(async function () {
      this.currentTest.token = await registerUserAndGetToken();
      this.currentTest.diaryUser = await User.findOne();
      const otherUser = await User.create(validUsers[0]);
      for (const diary of TestDiaries.validDiaries.slice(0, 5)) {
        await Diary.create({
          ...diary,
          userId: this.currentTest.diaryUser._id,
        });
      };
      for (const diary of TestDiaries.validDiaries.slice(5, 10)) {
        await Diary.create({
          ...diary,
          userId: otherUser._id,
        });
      };
    });

    it("should return all user diaries if logged in", async function () {
      const res = await request(app)
        .get("/api/diaries")
        .set("Authorization", this.test.token);

      expect(res.status).to.equal(200);
      expect(res.body.length).to.equal(5);
    });
  });
  describe("GET /:id", () => {
    it("should return a diary if authorised", async function () {
      const token = await registerUserAndGetToken();
      const user = await User.findOne();
      const diary = await Diary.create({
        ...TestDiaries.validDiaries[0],
        userId: user._id,
      });
      const res = await request(app)
        .get("/api/diaries/" + diary._id)
        .set("Authorization", token);

      expect(res.status).to.equal(200);
      expect(res.body).to.have.property(
        "title",
        TestDiaries.validDiaries[0].title
      );
    });

    it("should return 401 if not authorised", async function () {
      const token = await registerUserAndGetToken();
      const user = await User.create(validUsers[0]);
      const diary = await Diary.create({
        ...TestDiaries.validDiaries[0],
        userId: user._id,
      });
      const res = await request(app)
        .get("/api/diaries/" + diary._id)
        .set("Authorization", token);

      expect(res.status).to.equal(401);
    });

    it("should return 404 if diary does not exist", async function () {
      const token = await registerUserAndGetToken();
      
      const res = await request(app)
        .get("/api/diaries/610d3757fc13ae5d51000514")
        .set("Authorization", token);

      expect(res.status).to.equal(404);
    });

    it("should return 401 if not logged in at all", async function () {
      const user = await User.create(validUsers[0]);
      const diary = await Diary.create({
        ...TestDiaries.validDiaries[0],
        userId: user._id,
      });
      const res = await request(app).get("/api/diaries/" + diary._id);

      expect(res.status).to.equal(401);
    });
  });

  describe("POST /", () => {
    it("should return new diary with valid data and authorisation", async function () {
      const token = await registerUserAndGetToken();
      const diary = TestDiaries.validDiaries[0];
      const res = await request(app)
        .post("/api/diaries/")
        .set("Authorization", token)
        .send(diary);

      expect(res.status).to.equal(200);
      expect(res.body).to.have.property("title", diary.title);
    });

    it("should return 422 with invalid data", async function () {
      const token = await registerUserAndGetToken();
      const diary = Object.assign({}, TestDiaries.validDiaries[0]);
      delete diary.title;
      const res = await request(app)
        .post("/api/diaries/")
        .set("Authorization", token)
        .send(diary);

      expect(res.status).to.equal(422);
    });

    it("should return 401 with no authorisation", async function () {
      const diary = TestDiaries.validDiaries[0];
      const res = await request(app).post("/api/diaries/").send(diary);

      expect(res.status).to.equal(401);
    });
  });

  describe("PUT /:id", () => {
    it("should return updated diary with valid authorisation", async function () {
      const token = await registerUserAndGetToken();
      const user = await User.findOne();
      const diary = await Diary.create({
        ...TestDiaries.validDiaries[0],
        userId: user._id,
      });

      const res = await request(app)
        .put("/api/diaries/" + diary._id)
        .set("Authorization", token)
        .send({ title: "Updated" });

      expect(res.status).to.equal(200);
      expect(res.body).to.have.property("title", "Updated");
    });

    it("should return 401 with invalid authorisation", async function () {
      const token = await registerUserAndGetToken();
      const user = await User.create(validUsers[0]);
      const diary = await Diary.create({
        ...TestDiaries.validDiaries[0],
        userId: user._id,
      });

      const res = await request(app)
        .put("/api/diaries/" + diary._id)
        .set("Authorization", token)
        .send({ title: "Updated" });

      expect(res.status).to.equal(401);
    });
  });

  describe("DELETE /:id", () => {
    it("should delete and return a diary", async function () {
      const token = await registerUserAndGetToken();
      const user = await User.findOne();
      const diary = await Diary.create({
        ...TestDiaries.validDiaries[0],
        userId: user._id,
      });

      const diaries = await Diary.find();

      const res = await request(app)
        .delete("/api/diaries/" + diary._id)
        .set("Authorization", token);
      
      const newDiaries = await Diary.find();

      expect(res.status).to.equal(200);
      expect(res.body).to.have.property("_id", diary._id.toString());
      expect(newDiaries.length).to.equal(diaries.length - 1);
    });

    it("should return 404 if diary not found", async function () {
      const token = await registerUserAndGetToken();

      const res = await request(app)
        .delete("/api/diaries/610d3757fc13ae5d51000514")
        .set("Authorization", token);
      
      expect(res.status).to.equal(404);
    });

    it("should return 401 if not authorized", async function () {
      const token = await registerUserAndGetToken();
      const user = await User.create(validUsers[0]);
      const diary = await Diary.create({
        ...TestDiaries.validDiaries[0],
        userId: user._id,
      });

      const diaries = await Diary.find();

      const res = await request(app)
        .delete("/api/diaries/" + diary._id)
        .set("Authorization", token);
      
      const newDiaries = await Diary.find();

      expect(res.status).to.equal(401);
      expect(newDiaries.length).to.equal(diaries.length);
    });
  });
});
