const DiaryEntry = require("../../models/DiaryEntry.model");
const Diary = require("../../models/Diary.model");
const User = require("../../models/User.model");
const request = require("supertest");
const app = require("../../app");
const {
  registerUserAndGetToken,
} = require("../helpers/user_and_session.testhelper");
const { validUsers } = require("../datasets/user.testdata");
const { validDiaries } = require("../datasets/diary.testdata");
const { validDiaryEntries } = require("../datasets/diaryentry.testdata");
const { expect } = require("chai");
const Flashcard = require("../../models/Flashcard.model");

const makeEntries = async function (context) {
  context.currentTest.token = await registerUserAndGetToken();
  context.currentTest.user = await User.findOne();
  const otherUser = await User.create(validUsers[0]);
  for (const diary of validDiaries.slice(0, 2)) {
    await Diary.create({
      ...diary,
      userId: context.currentTest.user._id,
    });
  }
  for (const diary of validDiaries.slice(2, 4)) {
    await Diary.create({
      ...diary,
      userId: otherUser._id,
    });
  }
  const diaries = await Diary.find();
  for (let i = 0; i < diaries.length; i++) {
    const diary = diaries[i];
    await DiaryEntry.create(
      {
        ...validDiaryEntries[i * 2],
        userId: diary.userId,
        diaryId: diary._id,
      },
      {
        ...validDiaryEntries[i * 2 + 1],
        userId: diary.userId,
        diaryId: diary._id,
      }
    );
  }
  context.currentTest.myEntry = await DiaryEntry.findOne({
    userId: context.currentTest.user._id,
  });
  context.currentTest.otherEntry = await DiaryEntry.findOne({
    userId: otherUser._id,
  });
};

// From: https://stackoverflow.com/a/53434457
const wait = ms => new Promise(resolve => setTimeout(resolve, ms));

describe("api/entries", () => {
  describe("GET /", () => {
    beforeEach(async function () {
      await makeEntries(this);
    });

    it("should return all user diary entries if logged in", async function () {
      const res = await request(app)
        .get("/api/entries")
        .set("Authorization", this.test.token);

      expect(res.status).to.equal(200);
      expect(res.body.length).to.equal(4);
    });

    it("should return 401 if not logged in", async function () {
      const res = await request(app).get("/api/entries");

      expect(res.status).to.equal(401);
    });
  });

  describe("GET /:id", () => {
    beforeEach(async function () {
      await makeEntries(this);
    });

    it("should return diary entry if authorized and logged in", async function () {
      const res = await request(app)
        .get("/api/entries/" + this.test.myEntry._id)
        .set("Authorization", this.test.token);

      expect(res.status).to.equal(200);
      expect(res.body).to.have.property("content", this.test.myEntry.content);
    });

    it("should return 401 if not authorized (other user's entry)", async function () {
      const res = await request(app)
        .get("/api/entries/" + this.test.otherEntry._id)
        .set("Authorization", this.test.token);

      expect(res.status).to.equal(401);
    });

    it("should return 401 entry if not logged in", async function () {
      const res = await request(app).get(
        "/api/entries/" + this.test.myEntry._id
      );

      expect(res.status).to.equal(401);
    });

    it("should return 404 if entry does not exist", async function () {
      const res = await request(app)
        .get("/api/entries/610d3757fc13ae5d51000514")
        .set("Authorization", this.test.token);

      expect(res.status).to.equal(404);
    });
  });

  describe("PUT /:id", () => {
    beforeEach(async function () {
      await makeEntries(this);
    });

    it("should return updated entry with valid data and auth", async function () {
      const res = await request(app)
        .put("/api/entries/" + this.test.myEntry._id)
        .set("Authorization", this.test.token)
        .send({ location: "Updated" });

      expect(res.status).to.equal(200);
      expect(res.body).to.have.property("location", "Updated");
    });

    it("should return 401 with invalid auth", async function () {
      const res = await request(app)
        .put("/api/entries/" + this.test.myEntry._id)
        .send({ location: "Updated" });

      expect(res.status).to.equal(401);
    });
  });

  describe("DELETE /:id", () => {
    beforeEach(async function () {
      await makeEntries(this);
    });

    it("should delete and return a diary entry", async function () {
      const entriesBeforeDelete = await DiaryEntry.find();

      const res = await request(app)
        .delete("/api/entries/" + this.test.myEntry._id)
        .set("Authorization", this.test.token);

      const entriesAfterDelete = await DiaryEntry.find();

      expect(res.status).to.equal(200);
      expect(res.body).to.have.property(
        "_id",
        this.test.myEntry._id.toString()
      );
      expect(entriesAfterDelete.length).to.equal(
        entriesBeforeDelete.length - 1
      );
    });

    it("should return 401 with invalid auth (user is not entry owner)", async function () {
      const entriesBeforeDelete = await DiaryEntry.find();

      const res = await request(app)
        .delete("/api/entries/" + this.test.otherEntry._id)
        .set("Authorization", this.test.token);

      const entriesAfterDelete = await DiaryEntry.find();

      expect(res.status).to.equal(401);
      expect(entriesAfterDelete.length).to.equal(entriesBeforeDelete.length);
    });

    it("should return 404 if entry not found", async function () {
      const entriesBeforeDelete = await DiaryEntry.find();
      const res = await request(app)
        .delete("/api/entries/610d3757fc13ae5d51000514")
        .set("Authorization", this.test.token);

      const entriesAfterDelete = await DiaryEntry.find();
      expect(res.status).to.equal(404);
      expect(entriesAfterDelete.length).to.equal(entriesBeforeDelete.length);
    });
  });
});

describe("POST /api/diaries/:id/newentry", () => {
  beforeEach(async function () {
    this.currentTest.token = await registerUserAndGetToken();
    const user = await User.findOne();
    this.currentTest.diary = await Diary.create({
      ...validDiaries[0],
      userId: user._id,
    });
  });

  it("should return new entry with valid data and auth", async function () {
    const entry = validDiaryEntries[0];
    const res = await request(app)
      .post("/api/diaries/" + this.test.diary._id + "/newentry")
      .set("Authorization", this.test.token)
      .send(entry);

    expect(res.status).to.equal(200);
    expect(res.body).to.have.property("content", entry.content);
    expect(res.body).to.have.property(
      "diaryId",
      this.test.diary._id.toString()
    );
  });

  it("should create new flashcards for new entry with makeCards flag", async function () {
    this.timeout(10000);
    const entry = validDiaryEntries[0];
    entry["makeCards"] = true;
    const flashcardsBefore = await Flashcard.find();
    const res = await request(app)
      .post("/api/diaries/" + this.test.diary._id + "/newentry")
      .set("Authorization", this.test.token)
      .send(entry);
    await wait(3000);
    const flashcardsAfter = await Flashcard.find();
    const flashcard = await Flashcard.findOne({ diaryEntryId: res.body._id });

    expect(flashcardsBefore.length).to.be.lessThan(flashcardsAfter.length);
    expect(flashcard.diaryId.toString()).to.equal(this.test.diary._id.toString());
  });

  it("should return 422 with invalid data", async function () {
    const entry = Object.assign({}, validDiaryEntries[0]);
    delete entry.content;
    const res = await request(app)
      .post("/api/diaries/" + this.test.diary._id + "/newentry")
      .set("Authorization", this.test.token)
      .send(entry);

    expect(res.status).to.equal(422);
  });

  it("should return 401 with no auth", async function () {
    const entry = validDiaryEntries[0];
    const res = await request(app)
      .post("/api/diaries/" + this.test.diary._id + "/newentry")
      .send(entry);

    expect(res.status).to.equal(401);
  });
});
