const DiaryEntry = require("../../models/DiaryEntry.model");
const Diary = require("../../models/Diary.model");
const User = require("../../models/User.model");
const request = require("supertest");
const app = require("../../app");
const {
  registerUserAndGetToken,
} = require("../helpers/user_and_session.testhelper");
const { validUsers } = require("../datasets/user.testdata");
const { validDiaries } = require("../datasets/diary.testdata");
const { validDiaryEntries } = require("../datasets/diaryentry.testdata");
const { expect } = require("chai");
const Flashcard = require("../../models/Flashcard.model");
const { validFlashcards } = require("../datasets/flashcard.testdata");

// From: https://stackoverflow.com/a/53434457
const wait = ms => new Promise(resolve => setTimeout(resolve, ms));

const makeFlashcards = async function (context) {
  context.currentTest.token = await registerUserAndGetToken();
  context.currentTest.user = await User.findOne();
  const otherUser = await User.create(validUsers[0]);
  const myDiary = await Diary.create({
    ...validDiaries[0],
    userId: context.currentTest.user._id,
  });
  const otherDiary = await Diary.create({
    ...validDiaries[1],
    userId: otherUser._id,
  });
  const myEntry = await DiaryEntry.create({
    ...validDiaryEntries[0],
    userId: myDiary.userId,
    diaryId: myDiary._id,
  });
  await DiaryEntry.create({
    ...validDiaryEntries[0],
    userId: otherDiary.userId,
    diaryId: otherDiary._id,
  });
  const entries = await DiaryEntry.find();
  for (let i = 0; i < entries.length; i++) {
    const entry = entries[i];
    await Flashcard.create(
      {
        ...validFlashcards[i * 4],
        userId: entry.userId,
        diaryId: entry.diaryId,
        diaryEntryId: entry._id,
        entryDate: entry.date,
      },
      {
        ...validFlashcards[i * 4 + 1],
        userId: entry.userId,
        diaryId: entry.diaryId,
        diaryEntryId: entry._id,
        entryDate: entry.date,
      },
      {
        ...validFlashcards[i * 4 + 2],
        userId: entry.userId,
        diaryId: entry.diaryId,
        diaryEntryId: entry._id,
        entryDate: entry.date,
      },
      {
        ...validFlashcards[i * 4 + 3],
        userId: entry.userId,
        diaryId: entry.diaryId,
        diaryEntryId: entry._id,
        entryDate: entry.date,
      }
    );
  }
  context.currentTest.myFlashcard = await Flashcard.findOne({
    userId: context.currentTest.user._id,
  });
  context.currentTest.otherFlashcard = await Flashcard.findOne({
    userId: otherUser._id,
  });
  context.currentTest.myEntry = myEntry;
};

describe("api/flashcards", () => {
  beforeEach(async function () {
    await makeFlashcards(this);
  });
  describe("GET /", () => {
    it("should return all user flashcards if logged in", async function () {
      const res = await request(app)
        .get("/api/flashcards")
        .set("Authorization", this.test.token);

      expect(res.status).to.equal(200);
      expect(res.body.length).to.equal(4);
    });

    it("should return 401 if not logged in", async function () {
      const res = await request(app).get("/api/flashcards");

      expect(res.status).to.equal(401);
    });
  });

  describe("GET /:id", () => {
    it("should return diary entry if authorized and logged in", async function () {
      const res = await request(app)
        .get("/api/flashcards/" + this.test.myFlashcard._id)
        .set("Authorization", this.test.token);

      expect(res.status).to.equal(200);
      expect(res.body).to.have.property(
        "frontSide",
        this.test.myFlashcard.frontSide
      );
    });

    it("should return 401 if not authorized (other user's flashcard)", async function () {
      const res = await request(app)
        .get("/api/flashcards/" + this.test.otherFlashcard._id)
        .set("Authorization", this.test.token);

      expect(res.status).to.equal(401);
    });

    it("should return 401 flashcard if not logged in", async function () {
      const res = await request(app).get(
        "/api/flashcards/" + this.test.myFlashcard._id
      );

      expect(res.status).to.equal(401);
    });

    it("should return 404 if flashcard does not exist", async function () {
      const res = await request(app)
        .get("/api/flashcards/610d3757fc13ae5d51000514")
        .set("Authorization", this.test.token);

      expect(res.status).to.equal(404);
    });
  });

  describe("PUT /:id", () => {
    it("should return updated flashcard with valid data and auth", async function () {
      const res = await request(app)
        .put("/api/flashcards/" + this.test.myFlashcard._id)
        .set("Authorization", this.test.token)
        .send({ frontSide: "Updated" });

      expect(res.status).to.equal(200);
      expect(res.body).to.have.property("frontSide", "Updated");
    });

    it("should return 401 with invalid auth", async function () {
      const res = await request(app)
        .put("/api/flashcards/" + this.test.myFlashcard._id)
        .send({ frontSide: "Updated" });

      expect(res.status).to.equal(401);
    });
  });

  describe("DELETE /:id", () => {
    it("should delete and return a flashcard", async function () {
      const flashcardsBeforeDelete = await Flashcard.find();

      const res = await request(app)
        .delete("/api/flashcards/" + this.test.myFlashcard._id)
        .set("Authorization", this.test.token);

      const flashcardsAfterDelete = await Flashcard.find();

      expect(res.status).to.equal(200);
      expect(res.body).to.have.property(
        "_id",
        this.test.myFlashcard._id.toString()
      );
      expect(flashcardsAfterDelete.length).to.equal(
        flashcardsBeforeDelete.length - 1
      );
    });

    it("should return 401 with invalid auth (user is not flashcard owner)", async function () {
      const flashcardsBeforeDelete = await Flashcard.find();

      const res = await request(app)
        .delete("/api/flashcards/" + this.test.otherFlashcard._id)
        .set("Authorization", this.test.token);

      const flashcardsAfterDelete = await Flashcard.find();

      expect(res.status).to.equal(401);
      expect(flashcardsAfterDelete.length).to.equal(
        flashcardsBeforeDelete.length
      );
    });

    it("should return 404 if flashcard not found", async function () {
      const flashcardsBeforeDelete = await Flashcard.find();
      const res = await request(app)
        .delete("/api/flashcards/610d3757fc13ae5d51000514")
        .set("Authorization", this.test.token);

      const flashcardsAfterDelete = await Flashcard.find();
      expect(res.status).to.equal(404);
      expect(flashcardsAfterDelete.length).to.equal(
        flashcardsBeforeDelete.length
      );
    });
  });
});

describe("POST /api/entries/:id/newflashcard", () => {
  beforeEach(async function () {
    await makeFlashcards(this);
  });
  it("should return new flashcard with valid data and auth", async function () {
    const flashcard = validFlashcards[0];
    const res = await request(app)
      .post("/api/entries/" + this.test.myEntry._id + "/newflashcard")
      .set("Authorization", this.test.token)
      .send(flashcard);

    expect(res.status).to.equal(200);
    expect(res.body).to.have.property("frontSide", flashcard.frontSide);
    expect(res.body).to.have.property(
      "diaryEntryId",
      this.test.myEntry._id.toString()
    );
  });

  it("should return 422 with invalid data", async function () {
    const flashcard = Object.assign({}, validFlashcards[0]);
    delete flashcard.frontSide;
    const res = await request(app)
      .post("/api/entries/" + this.test.myEntry._id + "/newflashcard")
      .set("Authorization", this.test.token)
      .send(flashcard);

    expect(res.status).to.equal(422);
  });

  it("should return 401 with no auth", async function () {
    const flashcard = validFlashcards[0];
    const res = await request(app).post("/api/entries/" + this.test.myEntry._id + "/newflashcard").send(flashcard);

    expect(res.status).to.equal(401);
  });
});
