const User = require("../../models/User.model");
const request = require("supertest");
const app = require("../../app");
const { expect } = require("chai");
const TestUsers = require("../datasets/user.testdata");
const {
  registerUserAndGetToken,
} = require("../helpers/user_and_session.testhelper");

describe("api/users", () => {
  describe("GET /:id", () => {
    it("should return a user if a valid ID is passed", async function () {
      const user = new User(TestUsers.testUser);
      await user.save();
      const res = await request(app).get("/api/users/" + user._id);
      expect(res.status).to.equal(200);
      expect(res.body).to.have.property("username", "testuser");
    });

    it("should return 400 if invalid objectId is passed", async function () {
      const res = await request(app).get("/api/users/9");
      expect(res.status).to.equal(400);
    });

    it("should return 404 if a valid but non-existent objectId is passed", async function () {
      const res = await request(app).get("/api/users/610d3757fc13ae5d51000514");
      expect(res.status).to.equal(404);
    });
  });

  describe("GET /current", () => {
    beforeEach(async function () {
      this.currentTest.token = await registerUserAndGetToken();
    });

    it("should return user when logged in", async function () {
      const res = await request(app)
        .get("/api/users/current")
        .set("Authorization", this.test.token);
      expect(res.status).to.equal(200);
      expect(res.body).to.have.property("username");
    });

    it("should return 401 if bad token is sent", async function () {
      const res = await request(app)
        .get("/api/users/current")
        .set("Authorization", "Bearer bad_token");
      expect(res.status).to.equal(401);
    });

    it("should return 401 if no token is sent", async function () {
      const res = await request(app).get("/api/users/current");
      expect(res.status).to.equal(401);
    });
  });

  describe("POST /register", () => {
    it("should return user token when all data is valid", async function () {
      const res = await request(app)
        .post("/api/users/register")
        .send({
          ...TestUsers.testUser,
          confPassword: TestUsers.testUser.password,
        });
      expect(res.status).to.equal(200);
      expect(res.body).to.have.property("success", true);
      expect(res.body.token).to.include("Bearer ");
    });

    it("should return 409 when email already exists", async function () {
      const user = new User(TestUsers.testUser);
      await user.save();
      const res = await request(app)
        .post("/api/users/register")
        .send({
          ...TestUsers.testUser,
          username: "uniqueUsername",
          confPassword: TestUsers.testUser.password,
        });
      expect(res.status).to.equal(409);
    });

    it("should return 409 when username already exists", async function () {
      const user = new User(TestUsers.testUser);
      await user.save();
      const res = await request(app)
        .post("/api/users/register")
        .send({
          ...TestUsers.testUser,
          email: "unique@example.com",
          confPassword: TestUsers.testUser.password,
        });
      expect(res.status).to.equal(409);
    });
  });

  describe('"POST /login', () => {
    beforeEach(async function () {
      this.currentTest.token = await registerUserAndGetToken();
    });

    it("should return user token when all data is valid", async function () {
      const res = await request(app).post("/api/users/login").send({
        email: TestUsers.testUser.email,
        password: TestUsers.testUser.password,
      });
      expect(res.status).to.equal(200);
      expect(res.body).to.have.property("success", true);
      expect(res.body.token).to.include("Bearer ");
    });

    it("should return 404 if user not found", async function () {
      const res = await request(app).post("/api/users/login").send({
        email: "notauser@example.com",
        password: "notauser",
      });
      expect(res.status).to.equal(404);
    });

    it("should return 401 when credentials are incorrect", async function () {
      const resBadPassword = await request(app).post("/api/users/login").send({
        email: TestUsers.testUser.email,
        password: "wrongpassword",
      });
      expect(resBadPassword.status).to.equal(401);
    });
  });

  describe("PUT /:id", () => {
    beforeEach(async function () {
      this.currentTest.token = await registerUserAndGetToken();
    });

    it("should successfully update user information with valid data", async function () {
      const newUserData = {
        email: "newemail@example.com",
        firstName: "Newname",
      };
      const currentUserRes = await request(app)
        .get("/api/users/current")
        .set("Authorization", this.test.token);
      const currentUserId = currentUserRes.body._id;
      const res = await request(app)
        .put("/api/users/" + currentUserId)
        .set("Authorization", this.test.token)
        .send(newUserData);

      expect(res.status).to.equal(200);
      expect(res.body).to.have.property("email", "newemail@example.com");
      expect(res.body).to.have.property("firstName", "Newname");
    });

    it("should return 401 if logged in user is not the target user of update", async function () {
      const newUserData = {
        email: "newemail@example.com",
        firstName: "Newname",
      };
      const user = await new User(TestUsers.validUsers[0]).save();
      const res = await request(app)
        .put("/api/users/" + user._id)
        .set("Authorization", this.test.token)
        .send(newUserData);

      expect(res.status).to.equal(401);
    });
  });
});
